#pragma once

#include "grade_boundary_step.hpp"
#include "points.hpp"

#include <compare>

struct GradeBoundary {

    GradeBoundary() = default;
    GradeBoundary(Points p, GradeBoundaryStepSize stepSize,
                  GradeBoundaryRounding roundingMode) noexcept;
    constexpr explicit GradeBoundary(const int multiple) noexcept
        : mMultiple(multiple) {}

    [[nodiscard]] auto
    operator<=>(const GradeBoundary &) const noexcept = default;

    int mMultiple = 0;
};
[[nodiscard]] constexpr bool
boundaryGreater(const GradeBoundary &b, const Points p,
                const GradeBoundaryStepSize stepSize) noexcept {
    return static_cast<double>(b.mMultiple) * as<double>(stepSize) > p.p;
}

[[nodiscard]] constexpr Points
toPoints(const GradeBoundary b, const GradeBoundaryStepSize stepSize) noexcept {
    return Points{static_cast<double>(b.mMultiple) * as<double>(stepSize)};
}

[[nodiscard]] constexpr double operator/(const GradeBoundary &a,
                                         const GradeBoundary &b) noexcept {
    return static_cast<double>(a.mMultiple) / static_cast<double>(b.mMultiple);
}
[[nodiscard]] constexpr GradeBoundary
operator+(const GradeBoundary &a, const GradeBoundary &b) noexcept {
    return GradeBoundary{a.mMultiple + b.mMultiple};
}

[[nodiscard]] GradeBoundary
roundMultiple(double nrMultiple, GradeBoundaryRounding roundingMode) noexcept;

// Simplify grade boundary construction
struct GradeBoundaryFactory {
    GradeBoundaryFactory(GradeBoundaryStepSize stepSize,
                         GradeBoundaryRounding roundingMode) noexcept;

    [[nodiscard]] GradeBoundary operator()(Points p) const noexcept;

    GradeBoundaryStepSize mStepSize;
    GradeBoundaryRounding mRoundingMode;
};
